# -------------------------------------------------------------------------------------
# flo::plugin::Account::TicketSender
# -------------------------------------------------------------------------------------
#
#       Author : Steve Purkis <spurkis@mkdoc.com>
#    Copyright : (c) 2002, MKDoc Holdings Ltd.
#      Version : $Id: TicketSender.pm,v 1.1.2.6 2003/01/20 13:13:51 jhiver Exp $
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
# -------------------------------------------------------------------------------------

=head1 NAME

flo::plugin::Account::TicketSender - abstract Account plugin class

=head1 SYNOPSIS

 # inheritance
 use flo::plugin::Account::TicketSender;
 use base qw/ flo::plugin::Account::TicketSender /;

 # execute this plugin if activated:
 flo::plugin::Account::Details -> main();

=head1 DESCRIPTION

This abstract class defines generic methods for Account::* plugins that
send ticket emails.

=cut


package flo::plugin::Account::TicketSender;

use flo::plugin::Account::Generic;
use flo::Record::Ticket;

use MKDoc::Config;

use Carp;
use Time::ParseDate;

use strict;
use 5.008_000;
use utf8;



# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 INHERITS FROM

B<flo::plugin::Account::Generic>

=cut

use base qw/ flo::plugin::Account::Generic /;




# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 CLASS VARIABLES

=head2 $VERSION

Version (from CVS revision string).

=cut

our $VERSION = (split(/ /, '$Revision: 1.1.2.6 $'))[1];



# -------------------------------------------------------------------------------------

=head2 $DEBUG

Boolean switch controlling debug printing to STDERR.

=cut

our $DEBUG = 0;



# -------------------------------------------------------------------------------------

=head2 $EXPIRY_TIME

Ammount of time before generated tickets should expire.
Note that this is measured in system time.

=cut

our $EXPIRY_TIME = 7*24*60*60;       # One week, amended by Chris on 20/01/2003 


# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 STATIC METHODS

=head2 $ticket = $class->create_and_send_ticket( $email, $subject, $action [, $errors ] )

Error flags: 'ticket.save'
Returns ticket on success, undef on failure.

=cut

sub create_and_send_ticket ($$$)
{
    my $class   = shift || return;
    my $email   = shift || return;
    my $subject = shift || return;
    my $action  = shift || return;
    my $errors  = shift;

    # clear expired tickets:
    flo::Record::Ticket::clear_old_tickets_from_db;

    $class->check_for_existing_ticket($email, $errors) || return;

    # create a new ticket
    my $ticket = new flo::Record::Ticket;
    $ticket->generateId;
    $ticket->Email($email);
    $ticket->ExpiryDate(time + $EXPIRY_TIME);
    $ticket->Action($action);

    # save & send email
    if ($ticket->saveToDb())
    {
	# saved, ok to send
	$class->send_ticket_email($ticket, $subject, $errors);
    }
    else
    {
	warn "  !  error saving ticket to database!";
	$errors->set('ticket.save');
	return;
    }

    return $ticket;
}


# -------------------------------------------------------------------------------------

=head2 $class = $class->check_for_existing_ticket( $email, $errors )

Checks if there is *not* an existing ticket for this email and it is ok
to proceed.  On error, sets 'ticket.already.sent' to the expiry date of
the ticket.

Returns this class/obj, or undef on error.

=cut

sub check_for_existing_ticket {
    my $class  = shift;
    my $email  = shift;
    my $errors = shift;

    if (my $ticket = load flo::Record::Ticket(Email => $email)) {
	warn "  !  already sent to [$email] - expires on [".$ticket->ExpiryDate."]\n";
	$errors->set('ticket.already.sent', $ticket->ExpiryDate);
	return;
    }

    return $class;
}


# -------------------------------------------------------------------------------------

=head2 $doc = $class->send_ticket_email( $ticket, $subject [, $errors] )

Renders $class->email_template_file(), and sends a $subject email to
the user specified by C<$ticket->Email>.  The $ticket object is passed
to the template as 'ticket', so fields are accessible from Petal
with statements like:

    /ticket/ID, /ticket/ExpiryDate

See the L<flo::Record::Ticket> class for details on field names.

Errors are reported in the MKDoc::Error object, if passed.

Returns the true on success, or undef on error.

=cut

sub send_ticket_email ($$$)
{
    my $class   = shift || return;
    my $ticket  = shift || return;
    my $subject = shift || return;
    my $errors  = shift;

    $class->render_and_send_email(
				  To             => $ticket->Email,
				  Subject        => $subject,
				  Template       => $class->email_template_file,
				  ProcessingArgs => { ticket => $ticket },
				  Errors         => $errors || undef,
				 );
}



# -------------------------------------------------------------------------------------

=head2 $file = $class->email_template_file()

B<Abstract> method.
Returns the email template file associated with this class.

=cut

sub email_template_file ($)
{
    my $class = shift || return;
    confess "  !  $class does not override email_template_file()!";
}



# -------------------------------------------------------------------------------------

=head2 $time = $class->expiry_time( [ $time ] )

If C<$time> is set (a valid system time, may be negative), attempts
to set *default* expiry time $EXPIRY_TIME.

Gets expiry time from config, otherwise falls back on $EXPIRY_TIME.

Returns time, or undef on error.

=cut

sub expiry_time ($$)
{
    my $class = shift || return;
    my $time  = shift;

    if (defined $time)
    {
	$class->set_expiry_time($time) || return;
    }

    $time = $class->get_expiry_time_from_config || $EXPIRY_TIME;

    return $time;
}


# -------------------------------------------------------------------------------------

=head2 $class = $class->set_expiry_time( $time )

Sets default $EXPIRY_TIME to $time.
Returns this object/class, or undef on error.

=cut

sub set_expiry_time ($$)
{
    my $class = shift || return;
    my $time  = shift;

    if ($time =~ /^-?\d+$/)
    {
	warn "  +  setting default expiry_time to [$time]\n" if ($DEBUG);
	$EXPIRY_TIME = $time;
    }
    else
    {
	warn "  !  expiry time must be a number - got [$time]!\n";
	return undef;
    }

    return $class;
}



# -------------------------------------------------------------------------------------

=head2 $time = $class->get_expiry_time_from_config

Gets 'expiry_time' from 'flo.plugin.Account' config and parses it as
an offset (see Time::ParseDate for valid times).

Returns time, or undef on error.

=cut

sub get_expiry_time_from_config
{
    my $class        = shift || return;
    my $config       = new MKDoc::Config('flo.plugin.Account');
    my $expiry_time  = $config->get('ticket.expiry.time') || return;

    # we want an offset - so parse the date relative to
    # 1 second (we can't use 0, as parsedate() automatically
    # uses the current time)
    my ($time, $err) = parsedate($expiry_time, NOW => 1);

    if ($err)
    {
	warn "  !  warning: parsedate() encountered an error parsing " .
	     " the expiry time [$expiry_time] set in config: [$err]\n";
	return;
    }

    # be pedantic & subtract a second...
    return $time - 1;
}




# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 NOTES

n/a

=head1 AUTHOR

Steve Purkis <spurkis@mkdoc.com>

=head1 VERSION

$Id: TicketSender.pm,v 1.1.2.6 2003/01/20 13:13:51 jhiver Exp $

=head1 COPYRIGHT

Copyright (c) 2002, MKDoc Holdings Ltd.  All rights reserved.

=head1 SEE ALSO

L<flo::Record::Ticket>,
L<flo::plugin::Account::*> releated plugins,
L<flo::plugin::explorer::Admin> contains similar functionality

=cut



1;


__END__
